﻿###############################################################################
#                                                                             #
#                         Modify Mailbox for Office 365                       #
#                                  V3.9.8.6                                   #
#                       (C)2006-2021 KoXo Developpement                       #
#                                                                             #
###############################################################################
function log_message
{
Param($Header,$Message)	
$LogDate=Get-Date
$LogInfo="[$($LogDate)]: [$($Header)] $($Message)"
$LogInfo | Out-File -FilePath $LogFile -Encoding UTF8 -Append -Force
}
###############################################################################
function log_total_errors_count
{
$S="TotalErrorsCount=$($global:total_errors_count)"
$S | Out-File -FilePath $LogFile -Encoding UTF8 -Append -Force
}
###############################################################################
function log_errors_count
{
Param($ID)
$S="ErrorsCount[$($ID)]=$($global:errors_count)"
$S | Out-File -FilePath $LogFile -Encoding UTF8 -Append -Force
}
###############################################################################
function check_version
{
$S=Get-Host
if ($S.Version.Major -lt "2")
   {
   $S="Powershell's version must be greater or equal to 2.0 !"
   log_message "Version" $S
   $global:total_errors_count++
   write-host $S -foregroundcolor "red"
   write-host "Press 'Enter' to continue ..."
   Read-Host
   exit
   }
}
###############################################################################
function Banner
{
Param($MenuLines)
$MarginWidth=2
$MarginChar=" "
$MaxWidth=0;
$BorderTextColor="Magenta"
ForEach ($Line In $MenuLines)
        {
        If ($Line.Length -gt $MaxWidth) {$MaxWidth=$Line.Length}
        }
If ($MaxWidth -gt 0)
   {
   Write-Host "╔" -NoNewline -ForegroundColor $BorderTextColor
   For ($i=0; $i -lt $MaxWidth+2*$MarginWidth; $i++)
       {
       Write-Host "═" -NoNewline -ForegroundColor $BorderTextColor
       }
   Write-Host "╗" -ForegroundColor $BorderTextColor
   ForEach ($Line In $MenuLines)
           {
           Write-Host "║" -NoNewline -ForegroundColor $BorderTextColor
           Write-Host ("{0}{1}{2}" -f $MarginChar.PadLeft($MarginWidth),$Line.PadRight($MaxWidth),$MarginChar.PadLeft($MarginWidth)) -NoNewline -ForegroundColor White
           Write-Host "║" -ForegroundColor $BorderTextColor
           }
   Write-Host "╚" -NoNewline -ForegroundColor $BorderTextColor
   For ($i=0; $i -lt $MaxWidth+2*$MarginWidth; $i++)
       {
       Write-Host "═" -NoNewline -ForegroundColor $BorderTextColor
       }
   Write-Host "╝" -ForegroundColor $BorderTextColor
   }
}
###############################################################################
function welcome
{
$BannerText="Modifying user for Office 365 $($Version)",
            "(C) 2006-2021 KoXo Developpement"
Banner $BannerText
}
###############################################################################
function connect
{
Param([string] $MyURL,$MyName,$MyPassword,$ProxyEnabled,$ProxyUser,$ProxyPassword)
$error.Clear()
#Proxy
if ($ProxyEnabled)
   {
   if (![String]::IsNullorEmpty($ProxyUser))
      { 
      $SecuredPassword=ConvertTo-SecureString $ProxyPassword -AsPlainText -Force
      $ProxyCred=New-Object System.Management.Automation.PSCredential $ProxyUser, $SecuredPassword
      $PSSessionOption=New-PSSessionOption -ProxyAccessType IEConfig -ProxyAuthentication Negotiate -ProxyCredential $ProxyCred
      $S="Connection with Internet Explorer proxy parameters, User=$($ProxyUser)"
	  log_message "OK" $S
      }
      else 
      {
      $PSSessionOption=New-PSSessionOption -ProxyAccessType IEConfig -ProxyAuthentication Negotiate
      $S="Connection with Internet Explorer proxy parameters"
	  log_message "OK" $S
      }
   write-host $S -foregroundcolor "cyan"   
   }
# Set credential
if ( ([String]::IsNullorEmpty($MyPassword)) -or ([String]::IsNullorEmpty($MyName))  )
   {
   $Cred=Get-Credential
   }
   else
   {
   $SecuredPassword=ConvertTo-SecureString $MyPassword -AsPlainText -Force
   $Cred=New-Object System.Management.Automation.PSCredential $MyName, $SecuredPassword
   }
# Creating new runspace. 
if ($ProxyEnabled)
   {
   $Script:RS=New-PSSession -ConfigurationName microsoft.exchange -ConnectionUri $MyURL -Credential $Cred -Authentication Basic -AllowRedirection -SessionOption $PSSessionOption -ErrorAction SilentlyContinue
   }
   else
   {
   $Script:RS=New-PSSession -ConfigurationName microsoft.exchange -ConnectionUri $MyURL -Credential $Cred -Authentication Basic -AllowRedirection -ErrorAction SilentlyContinue
   }
if ([String]::IsNullorEmpty($error[0]))
   {
    $S="Runspace Creation was successful = $($MyURL)"
	log_message "OK" $S
	# Import-PSSession $Script:RS
   } 
   else 
   {
   $global:total_errors_count++
   $S="Runspace creation was unsuccessful to $($MyURL) || $($error[0])"
   log_message "ERROR" $S
   # Display error
   write-host $S -foregroundcolor "red"
   write-host "Press 'Enter' to continue ..."
   Read-Host   
   }   
 return $Cred
}
###############################################################################
function close
{
$error.Clear()
$Script:RS | Remove-PSSession -ErrorAction SilentlyContinue
if ([String]::IsNullorEmpty($error[0]))
	{
    $S="Successfully cleared Runspace"
	log_message "OK" $S
	}
	else
	{
	$global:total_errors_count++
    $S="Clearing Runspace was unsuccessful. Error = $($error[0])"
    log_message "ERROR" $S
	}
}
###############################################################################
## Install MSOnline Cmdlets 
function Install-MSOnlineCmdlets()
{
$Res=$false
$DotNET35Path="HKLM:\SOFTWARE\Microsoft\NET Framework Setup\NDP\v3.5"
$DotNet35Present=((Test-Path $DotNET35Path) -and ((Get-ItemProperty $DotNET35Path).Install -eq "1"))
if ($DotNet35Present)
   {
   # Trying to import module
   if (-not(Get-Module -Name "MSOnline")){Get-Module -ListAvailable | Where-Object {$_.name -eq "MSOnline"} | Import-Module}
   # Module is not present
   if (-not(Get-Module -Name "MSOnline"))
      {
      Write-Host "MSOnline Powershell module is not present !" -Foregroundcolor "Red"
      $wc=New-Object System.Net.WebClient
      # Sign-in Assistant
      $SigninAssistantPath="HKLM:\SOFTWARE\Microsoft\MSOIdentityCRL"
      $SigninAssistantPresent=((Test-Path $SigninAssistantPath) -and ((Get-ItemProperty $SigninAssistantPath).TargetDir -ne $null))
      $MyCulture="0x{0:X}" -f(Get-Culture).LCID
      if (!$SigninAssistantPresent)
         {
         if ((Get-WmiObject Win32_OperatingSystem).OSArchitecture.Contains("64"))
         	  # V7.250.4556.0
         	  {$Source="http://www.koxo.net/fichiers/msonline/msoidcli_64.msi"} ## {$Source="http://download.microsoft.com/download/A/7/5/A75FE163-6C03-4E44-8E81-70905727D8E6/msoidcli_64.msi"}
            else
            # V7.250.4556.0
            {$Source="http://www.koxo.net/fichiers/msonline/msoidcli_32.msi"} ## {$Source="http://download.microsoft.com/download/A/7/5/A75FE163-6C03-4E44-8E81-70905727D8E6/msoidcli_32.msi"}
         $Destination=$env:TEMP+"\_SigninAssistant_.msi"
         Write-Host "Trying to download ""Microsoft Online Sign-in Assistant"""
         Write-Host "Url=$($Source)"
         Write-Host "Please, wait ..." -ForegroundColor "Cyan"
         $wc.DownloadFile($Source,$Destination)
         $Install=Start-Process $Destination -Wait -PassThru
         if ($Install.ExitCode -eq 0)
            {
            $S="Microsoft Online Sign-in Assistant installation was OK"
            Write-Host $S -ForegroundColor "Green"
            Log_Message "OK" $S
            }
            else
            {
            $S="Installation failed for Microsoft Online Sign-in Assistant, code : $($Install.ExitCode)"
            Write-Host $S -ForegroundColor "Red"
            Log_Message "ERROR" $S
            } 
         $FileExists=Test-Path $Destination
         If ($FileExists) {Get-Item $Destination | Remove-Item} 
         }
      # MSOnline module
      $MSOnlinePath="HKLM:\SOFTWARE\Microsoft\MSOnlinePowerShell"
      $MSOnlinePresent=((Test-Path $MSOnlinePath) -and ((Get-ItemProperty $MSOnlinePath).InstallPath -ne $null))
      if (!$MSOnlinePresent)
         {
         if ((Get-WmiObject Win32_OperatingSystem).OSArchitecture.Contains("64"))
            {
            # 64 bits
            if ([int](Get-WmiObject Win32_OperatingSystem).Version.Split(".")[0] -lt 6)
               {
               # Windows 2003
               $Source="http://www.koxo.net/fichiers/msonline/AdministrationConfig-fr-x64-patched.msi"
               }
               else
               {
               # Windows 2008 and above
               # V1.1.166.0
               $Source="http://www.koxo.net/fichiers/msonline/AdministrationConfig_3-x64.msi" ## $Source="http://download.connect.microsoft.com/pr/AdministrationConfig_3.msi?t=3edbe1ec-9076-441a-ba1d-6a8ae147d3ea&e=1497200079&h=81bba22aebadb1efceb708c41c45fa85"
               }
            }
            else
            {
            # 32 bits
            if ([int](Get-WmiObject Win32_OperatingSystem).Version.Split(".")[0] -lt 6)
               {
               # Windows 2003
               # V1.0
               $Source="http://www.koxo.net/fichiers/msonline/AdministrationConfig-fr-x86-patched.msi"
               }
               else
               {
               # Windows 2008 and above
               # V1.0
               $Source="http://www.koxo.net/fichiers/msonline/AdministrationConfig-fr-x86.msi" ## $Source="http://go.microsoft.com/fwlink/?linkid=236298" ## $Source=$Source+"&clcid="+$MyCulture
               }
            }
         $Destination=$env:TEMP+"\_MSOnline_.msi"
         write-host "Trying to download ""Microsoft Online Powershell Module"""
         Write-Host "Url=$($Source)"
         Write-Host "Please, wait ..." -ForegroundColor "Cyan"
         $wc.DownloadFile($Source,$Destination)
         $Install=Start-Process $Destination -Wait -PassThru
         if ($Install.ExitCode -eq 0)
            {
            $S="Microsoft Online Powershell Module installation was OK"
            Write-Host $S -ForegroundColor "Green"
            Log_Message "OK" $S
            }
            else
            {
            $S="Installation failed for Microsoft Online Powershell Module, code : $($Install.ExitCode)"
            Write-Host $S -ForegroundColor "Red"
            Log_Message "ERROR" $S
            }
         $FileExists=Test-Path $Destination
         if ($FileExists) {Get-Item $Destination | Remove-Item} 
         }
      # Import MSOnline Module
      $Mod=Import-Module "MSOnline" -PassThru
      if ([string]::IsNullOrEmpty($Mod)) 
         {
         $S="MSOnline Powershell module is still not present !"
         Write-Host $S -Foregroundcolor "Red"
         Log_Message "ERROR" $S
         Write-Host "Press any key ..."
         Read-Host
         }
         else
         {
         $S="MSOnline V{0} Powershell module is now present and loaded !"
         Write-Host ($S -f $Mod.Version.ToString()) -Foregroundcolor "Green"
         Log_Message "OK" $S
         $Res=$true
         }
      }
      else
      {
      $Mod=Get-Module MSOnline
      $S="MSOnline V{0} Powershell module is present"
      Write-Host ($S -f $Mod.Version.ToString()) -Foregroundcolor "Green"
      $Res=$true
      }
  }
  else
  {
   $S="Please, install Framework .NET V3.5 first !"
   Write-Host $S -ForegroundColor "Red"
   Log_Message "ERROR" $S
   Write-Host "Press any key ..."
   Read-Host
  }
return $Res
}
###############################################################################
function distribution_group_exists
{
Param([string] $GroupId)
$error.Clear()
$Group=Invoke-Command -Session $Script:RS -ErrorAction SilentlyContinue -arg $GroupId {Param($Id) Get-DistributionGroup -Identity $Id}
return (![string]::IsNullOrEmpty($Group))
}
###############################################################################
function user_exists
{
Param([string] $email)
$error.Clear()
$User=Invoke-Command -Session $Script:RS -ErrorAction SilentlyContinue -ArgumentList $email {Param($id) Get-User -Identity $id}
return (![string]::IsNullOrEmpty($User))
}
###############################################################################
function get_koxo_security_group
{
Param([string] $grp)
$error.Clear()
$Group=Get-MsolGroup -SearchString $grp -GroupType "Security" | Where {$_.Description -cmatch "[KOXO]"}
return $Group
}
###############################################################################
###############################################################################
function modify_user
{
Param($UserEmail,
      $UserMustChangePassword,
      $UserHiddenFromAddressLists,
      $UserDisplayName,
      $UserTitle,$UserFirstName,$UserLastName,
      $UserInitials,
      $UserPhone,$UserMobilePhone,$UserFaxNumber,
      $UserWebPage,
      $UserDepartment,$UserOffice,$UserCompany,
      $UserEmail1,$UserEmail2,$UserEmail3,$UserEmail4,$UserEmail5,
      $UserNotes,
      $UserPicture,
      $MemberOf,
      $CustomAttribute1,$CustomAttribute2,$CustomAttribute3,$CustomAttribute4,
      $CustomAttribute5,$CustomAttribute6,$CustomAttribute7,$CustomAttribute8,
      $CustomAttribute9,$CustomAttribute10,$CustomAttribute11,$CustomAttribute12,
      $CustomAttribute13,$CustomAttribute14,$CustomAttribute15
      )
$UserIdentity=$UserEmail.Split("@")[0]   
if ($UserMustChangePassword -isnot[Boolean]) {$UserMustChangePassword=$false}
if ($UserHiddenFromAddressLists -isnot[Boolean]) {$UserHiddenFromAddressLists=$false}
$UserNotes=$UserNotes -replace ("/","`n")
$error.Clear()
# Remove added emails
$Id=Invoke-Command -Session $Script:RS -ErrorAction SilentlyContinue -arg $UserEmail {Param($id) get-mailbox -Identity $id}
ForEach ($Email in $Id.EmailAddresses)
        {
         If (($Email -clike "smtp:*") -Or ($Email -clike "SIP:*"))
            {
            Invoke-Command -Session $Script:RS -ErrorAction SilentlyContinue -Arg $UserEmail,$Email {Param($id,$em) set-mailbox -Identity $id -EmailAddresses @{remove=$em} }
            }
        }	
# Build the list of Proxy Email Addresses
$EmailAddresses=New-Object System.Collections.ArrayList
if (![String]::IsNullorEmpty($UserEmail))
   {
	$index=$EmailAddresses.Add("SIP:$($UserEmail)")
	$index=$EmailAddresses.Add("SMTP:$($UserEmail)")
	if (![String]::IsNullorEmpty($UserEmail1))
	   {
	   $index=$EmailAddresses.Add("smtp:$($UserEmail1)")
	   }
	if (![String]::IsNullorEmpty($UserEmail2))
	   {
	   $index=$EmailAddresses.Add("smtp:$($UserEmail2)")
	   }
	if (![String]::IsNullorEmpty($UserEmail3))
  	   {
	   $index=$EmailAddresses.Add("smtp:$($UserEmail3)")
   	   }
	if (![String]::IsNullorEmpty($UserEmail4))
  	   {
	   $index=$EmailAddresses.Add("smtp:$($UserEmail4)")
   	   }
	if (![String]::IsNullorEmpty($UserEmail5))
  	   {
	   $index=$EmailAddresses.Add("smtp:$($UserEmail5)")
   	   }
	}
# Add Proxy Email Addresses and Custom Attributes    
$error.Clear()
%{Invoke-Command -Session $Script:RS {param ($1,$2,$3,$4,$5,$6,$7,$8,$9,$10,$11,$12,$13,$14,$15,$16,$17,$18) set-mailbox -Identity $1 -EmailAddresses $2 -CustomAttribute1 $3 -CustomAttribute2 $4 -CustomAttribute3 $5 -CustomAttribute4 $6 -CustomAttribute5 $7 -CustomAttribute6 $8 -CustomAttribute7 $9 -CustomAttribute8 $10 -CustomAttribute9 $11 -CustomAttribute10 $12 -CustomAttribute11 $13 -CustomAttribute12 $14 -CustomAttribute13 $15 -CustomAttribute14 $16 -CustomAttribute15 $17 -HiddenFromAddressListsEnabled $18}-arg $UserEmail,$EmailAddresses,$CustomAttribute1,$CustomAttribute2,$CustomAttribute3,$CustomAttribute4,$CustomAttribute5,$CustomAttribute6,$CustomAttribute7,$CustomAttribute8,$CustomAttribute9,$CustomAttribute10,$CustomAttribute11,$CustomAttribute12,$CustomAttribute13,$CustomAttribute14,$CustomAttribute15,$UserHiddenFromAddressLists}>$Results    	
if ([String]::IsNullorEmpty($error[0]))
	{
	$S="Successfully update mailbox for $($UserDisplayName) ($($UserEmail))"
    write-host $S -ForegroundColor Green
    log_message "OK" $S
	}
	else
	{
	$global:errors_count++
    $S="Updating mailbox for $($UserDisplayName) ($($UserEmail)). Error = $($error[0])"
    write-host $S  -ForegroundColor Red
    log_message "ERROR" $S
    }
# Update User Misc. Attributes 1    
$error.Clear()
Set-MsolUser -UserPrincipalName $UserEmail -Title $UserTitle -DisplayName $UserDisplayName -FirstName $UserFirstName -LastName $UserLastName -PhoneNumber $UserPhone -MobilePhone $UserMobilePhone -Fax $UserFaxNumber -Department $UserDepartment -Office $UserOffice -ErrorAction SilentlyContinue
if ([String]::IsNullorEmpty($error[0]))
	{
	$S="Successfully update user's AzureAD informations for $($UserDisplayName) ($($UserEmail))"
    write-host $S -ForegroundColor Green
    log_message "OK" $S
	}
	else
	{
	$global:errors_count++
    $S="Updating user's AzureAD informations for $($UserDisplayName) ($($UserEmail)). Error = $($error[0])"
    write-host $S  -ForegroundColor Red
    log_message "ERROR" $S
    }
# Update User Misc. Attributes 2   
$error.Clear()
%{Invoke-Command -Session $Script:RS {param ($id,$initials,$webpage,$company,$notes,$resetpassword) set-user -Identity $id -Initials $initials -WebPage $webpage -Company $company -Notes $notes -ResetPasswordOnNextLogon $resetpassword}-arg $UserEmail,$UserInitials,$UserWebPage,$UserCompany,$UserNotes,$UserMustChangePassword}>$Results
if ([String]::IsNullorEmpty($error[0]))
	{
	$S="Successfully update user's Exchange informations for $($UserDisplayName) ($($UserEmail))"
    write-host $S  -ForegroundColor Green
    log_message "OK" $S
	}
	else
	{
	$global:errors_count++
    $S="Updating user's Exchange informations for $($UserDisplayName) ($($UserEmail)). Error = $($error[0])"
    write-host $S  -ForegroundColor Red
    log_message "ERROR" $S
    }
# Add membership to distribution groups
$Groups=$MemberOf.Split(",",[System.StringSplitOptions]::RemoveEmptyEntries);
ForEach ($Group in $Groups)
        {
        $error.Clear()
        %{Invoke-Command -Session $Script:RS -ErrorAction SilentlyContinue {param ($grp,$memb) add-distributiongroupmember -Identity $grp -Member $memb}-arg $Group,$UserIdentity}
        if ([String]::IsNullorEmpty($error[0]))
           {
           $S="Successfully add distribution group ""$($Group)"" for $($UserDisplayName) ($($UserEmail))"
           write-host $S
           log_message "OK" $S
           }
        }  
# Add membership to security groups
ForEach ($Group in $Groups)
        {
        $error.Clear()
        $SecurityGroup=get_koxo_security_group $Group
        if (![string]::IsNullOrEmpty($SecurityGroup)) 
           {	 
           $User=Get-MsolUser -UserPrincipalName $UserEmail
           Add-MsolGroupMember -GroupObjectId $SecurityGroup.ObjectId -GroupMemberType "User" -GroupMemberObjectId $User.ObjectId -ErrorAction SilentlyContinue
           if ([String]::IsNullorEmpty($error[0]))
              {
              $S="Successfully add security group ""$($SecurityGroup.DisplayName)"" for $($UserDisplayName) ($($UserEmail))"
              Write-Host $S
              log_message "OK" $S
              }
           }   
        }  
# Add user's picture or photo / must be smaller than 10KB
if (![String]::IsNullorEmpty($UserPicture))
   {
   If ((Test-Path $UserPicture) -eq $True)
   	  {
      $error.Clear()
      %{Invoke-Command -Session $Script:RS {param ($id,$picture) set-userphoto -Identity $id -PictureData ([System.IO.File]::ReadAllBytes($picture)) -Confirm:$false}-arg $UserEmail,$UserPicture}>$Results
      if ([String]::IsNullorEmpty($error[0]))
	       {
	       $FileSize=(Get-Item $UserPicture).Length / 1024
	       $S="Successfully update user's picture ""$($UserPicture)"" ({0:n1}KB) for $($UserDisplayName) ($($UserEmail))" -f $FileSize
         log_message "OK" $S
	       }
	       else
	       {
	       $global:errors_count++
         $FileSize=(Get-Item $UserPicture).Length / 1024
         $S="Updating user's picture ""$($UserPicture)"" ({0:n1}KB) for $($UserDisplayName) ($($UserEmail)). Error = $($error[0])" -f $FileSize
         log_message "ERROR" $S
         }
      }   
   }
}
###############################################################################
function display_user
{
Param ($UserDisplayName,$UserEmail)
Write-Host "User: """ -nonewline
Write-Host $UserDisplayName -foregroundcolor "Green" -nonewline
Write-Host """ Email: """ -nonewline
Write-Host $UserEmail -foregroundcolor "Green" -nonewline
Write-host """"	
}
###############################################################################
function exec
{
Param($UserEmail, 
      $UserMustChangePassword,$UserHiddenFromAddressLists,
      $UserDisplayName,$UserTitle,$UserFirstName,$UserLastName,$UserInitials, 
      $UserPhone,$UserMobilePhone,$UserFaxNumber, 
      $UserWebPage,$UserDepartment,$UserOffice,$UserCompany, 
      $UserEmail1,$UserEmail2,$UserEmail3,$UserEmail4,$UserEmail5, 
      $UserNotes, 
      $UserPicture,
      $MemberOf,
      $CustomAttribute1,$CustomAttribute2,$CustomAttribute3,
      $CustomAttribute4,$CustomAttribute5,$CustomAttribute6, 
      $CustomAttribute7,$CustomAttribute8,$CustomAttribute9,
      $CustomAttribute10,$CustomAttribute11,$CustomAttribute12,
      $CustomAttribute13,$CustomAttribute14,$CustomAttribute15
      )
$UserDisplayName=$UserDisplayName.Trim()
log_message "OFFICE365" "$($UserDisplayName) ($($UserEmail))"
if (user_exists($UserEmail))
   {
   display_user	$UserDisplayName $UserEmail
   modify_user $UserEmail $UserMustChangePassword $UserHiddenFromAddressLists $UserDisplayName $UserTitle $UserFirstName $UserLastName $UserInitials $UserPhone $UserMobilePhone $UserFaxNumber $UserWebPage $UserDepartment $UserOffice $UserCompany $UserEmail1 $UserEmail2 $UserEmail3 $UserEmail4 $UserEmail5 $UserNotes $UserPicture $MemberOf $CustomAttribute1 $CustomAttribute2 $CustomAttribute3 $CustomAttribute4 $CustomAttribute5 $CustomAttribute6 $CustomAttribute7 $CustomAttribute8 $CustomAttribute9 $CustomAttribute10 $CustomAttribute11 $CustomAttribute12 $CustomAttribute13 $CustomAttribute14 $CustomAttribute15 
   }
   else
   {
   $errors_count++
   $S="User ""$($UserEmail)"" does not exists"
   write-host $S -foregroundcolor "red"
   log_message "ERROR" $S
   }
log_errors_count($UserEmail)   
$global:total_errors_count=$global:total_errors_count+$global:errors_count
}
#
###############################################################################
#                                                                             #
#                                    Main                                     #
#                                                                             #
###############################################################################
#
$global:Version="V3.9.8.6"
#
# Office 365 params
#
$Office365Url="https://ps.outlook.com/powershell/"
$Office365LoginName="%OFFICE365_ADMIN%"
$Office365LoginPassword=[System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("%OFFICE365_BASE64_PASSWORD%"))
#
# Proxy parameters
#
$ProxyEnabled=$#BOOL{%PROXY_ENABLED%}
$ProxyUser="%PROXY_USER%"     
$ProxyPassword=[System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("%PROXY_BASE64_PASSWORD%"))
#
#
$LogFile=$MyInvocation.MyCommand.Definition 
$LogFile=$LogFile -replace(".ps1",".log")
$global:total_errors_count=0
$DoNotUseUserPhones=$#BOOL{%OFFICE365_DO_NOT_USE_USER_PHONES%}
welcome
check_version
$Credential=connect $Office365Url $Office365LoginName $Office365LoginPassword $ProxyEnabled $ProxyUser $ProxyPassword
if ($global:total_errors_count -eq 0)
   {
   if (Install-MSOnlineCmdlets)
      {
      Connect-MSolService -Credential $Credential
      if ($global:total_errors_count -eq 0)
         {  
         ### BEGIN_REPETITION ###
         $global:errors_count=0
         $UserMustChangePassword=$#BOOL{%OFFICE365_RESET_PASSWORD_ON_NEXT_LOGON%}
         $UserHiddenFromAddressLists=$#BOOL{%OFFICE365_USER_HIDDEN_FROM_ADDRESS_LISTS%}
         $UserEmail="%USER_EMAIL%"
         $UserPassword=[System.Text.Encoding]::ASCII.GetString([System.Convert]::FromBase64String("%USER_BASE64_PASSWORD%"))
         $UserDisplayName="%USER_DISPLAY_NAME%"
         $UserTitle="%USER_TITLE%"
         $UserFirstName="%USER_FIRST_NAME%"
         $UserLastName="%USER_LAST_NAME%"
         $UserInitials="%USER_INITIALS%"
         if ($DoNotUseUserPhones)
            { 
            $UserPhone=""
            $UserMobilePhone=""
            $UserFaxNumber=""
            }
	        else
	        {
            $UserPhone="%USER_PHONE%"
            $UserMobilePhone="%USER_MOBILE_PHONE%"
            $UserFaxNumber="%USER_FAX_NUMBER%"
	        }
         $UserWebPage="%USER_WEB_PAGE%"
         $UserDepartment="%PRIMARY_GROUP% / %SECONDARY_GROUP%"
         $UserOffice="%SECONDARY_GROUP%"
         $UserCompany="%COMPANY%"
         $UserEmail1="%USER_EMAILS[1]%"
         $UserEmail2="%USER_EMAILS[2]%"
         $UserEmail3="%USER_EMAILS[3]%"
         $UserEmail4="%USER_EMAILS[4]%"
         $UserEmail5="%USER_EMAILS[5]%"
         $UserPicture="%USER_ID_THUMBNAIL_PICTURE%"
         $CustomAttribute1 ="#LOWER{#REMOVE_SPACE{%PRIMARY_GROUP%}}"
         $CustomAttribute2 ="#LOWER{#REMOVE_SPACE{%SECONDARY_GROUP%}}"
         $CustomAttribute3 ="%USER_UNIQUE_ID%"
         $CustomAttribute4 ="%USER_BIRTHDATE%"
         $CustomAttribute5 ="%USER_OTHER_MAILBOXES%"
         $CustomAttribute6 =""
         $CustomAttribute7 =""
         $CustomAttribute8 =""
         $CustomAttribute9 =""
         $CustomAttribute10=""
         $CustomAttribute11=""
         $CustomAttribute12=""
         $CustomAttribute13=""
         $CustomAttribute14=""
         $CustomAttribute15=""
         $MemberOf="$($CustomAttribute1),$($CustomAttribute2)"
         exec $UserEmail $UserMustChangePassword $UserHiddenFromAddressLists $UserDisplayName $UserTitle $UserFirstName $UserLastName $UserInitials $UserPhone $UserMobilePhone $UserFaxNumber $UserWebPage $UserDepartment $UserOffice $UserCompany $UserEmail1 $UserEmail2 $UserEmail3 $UserEmail4 $UserEmail5 $UserNotes $UserPicture $MemberOf $CustomAttribute1 $CustomAttribute2 $CustomAttribute3 $CustomAttribute4 $CustomAttribute5 $CustomAttribute6 $CustomAttribute7 $CustomAttribute8 $CustomAttribute9 $CustomAttribute10 $CustomAttribute11 $CustomAttribute12 $CustomAttribute13 $CustomAttribute14 $CustomAttribute15 
         ### END_REPETITION ###
         }
         else
         {
         $global:total_errors_count++	
         }	
      }
      else
      {
      $S="MSOnline Powershell module cannot be loaded !"
      Write-Host $S -Foregroundcolor "Red"
      Log_Message "ERROR" $S
      $global:total_errors_count++	
      }	
   close
   }
log_total_errors_count
